<?php
/**
 * Copyright (c) 2012-2013 Jurian Sluiman.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *
 *   * Neither the names of the copyright holders nor the names of the
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * @package     SlmException
 * @author      Jurian Sluiman <jurian@juriansluiman.nl>
 * @copyright   2012-2013 Jurian Sluiman http://juriansluiman.nl.
 * @license     http://www.opensource.org/licenses/bsd-license.php  BSD License
 */

namespace SlmException;

use Zend\ModuleManager\Feature;
use Zend\EventManager\EventInterface;
use Zend\Http\Request as HttpRequest;

class Module implements
    Feature\AutoloaderProviderInterface,
    Feature\ServiceProviderInterface,
    Feature\ConfigProviderInterface,
    Feature\BootstrapListenerInterface
{
    public function getAutoloaderConfig()
    {
        return array(
            'Zend\Loader\StandardAutoloader' => array(
                'namespaces' => array(
                    __NAMESPACE__ => __DIR__ . '/src/' . __NAMESPACE__,
                ),
            ),
        );
    }

    public function getServiceConfig()
    {
        return include __DIR__ . '/config/services.config.php';
    }

    public function getConfig()
    {
        return include __DIR__ . '/config/module.config.php';
    }

    public function onBootstrap(EventInterface $e)
    {
        $app    = $e->getParam('application');
        $sm     = $app->getServiceManager();
        $config = $sm->get('config');
        $config = $config['slm_exception'];

        if (true === $config['enable_markers']) {
            $this->attachExceptionListeners($e);
        }
        if (true === $config['enable_logging']) {
            $this->attachExceptionLogging($e);
        }
        if (true === $config['enable_messages']) {
            $this->attachExceptionMessages($e);
        }
        if (true === $config['enable_problem_api']) {
            $this->attachProblemApiListeners($e);
        }
    }

    protected function attachExceptionListeners(EventInterface $e)
    {
        $app = $e->getParam('application');
        $sm  = $app->getServiceManager();
        $em  = $app->getEventManager();

        // Remove the default error strategies to not interfere
        if($app->getRequest() instanceof HttpRequest) {
            // Remove the default error strategies to not interfere
            $strategy = $sm->get('Zend\Mvc\View\Http\RouteNotFoundStrategy');
            $strategy->detach($em);

            $strategy = $sm->get('Zend\Mvc\View\Http\ExceptionStrategy');
            $strategy->detach($em);

            // Attach the new strategy
            $strategy = $sm->get('SlmException\Mvc\View\Http\ExceptionStrategy');
            $strategy->attach($em);
        }
    }

    protected function attachExceptionLogging(EventInterface $e)
    {
        // @todo Implement logging strategy
    }

    protected function attachExceptionMessages(EventInterface $e)
    {
        // @todo Implement user defined error messages
    }

    protected function attachProblemApiListeners(EventInterface $e)
    {
        $app = $e->getApplication();
        $em  = $app->getEventManager();

        $listener = new Mvc\JsonProblemApiListener;
        $listener->attach($em);
    }
}
